package cn.com.infosec.crypto.modes;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.Mac;
import cn.com.infosec.crypto.macs.CMac;
import cn.com.infosec.crypto.params.AEADParameters;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class EAXBlockCipher implements AEADBlockCipher {
   private static final byte nTAG = 0;
   private static final byte hTAG = 1;
   private static final byte cTAG = 2;
   private SICBlockCipher cipher;
   private boolean forEncryption;
   private int blockSize;
   private Mac mac;
   private byte[] nonceMac;
   private byte[] associatedTextMac;
   private byte[] macBlock;
   private int macSize;
   private byte[] bufBlock;
   private int bufOff;

   public EAXBlockCipher(BlockCipher cipher) {
      this.blockSize = cipher.getBlockSize();
      CMac var2;
      CMac var10007 = var2 = new CMac;
      var10007.<init>(cipher);
      this.mac = var10007;
      int var10005 = this.blockSize;
      this.macBlock = new byte[var10005];
      this.bufBlock = new byte[var10005 * 2];
      this.associatedTextMac = new byte[var2.getMacSize()];
      this.nonceMac = new byte[this.mac.getMacSize()];
      this.cipher = new SICBlockCipher(cipher);
   }

   private void calculateMac() {
      byte[] var1 = new byte[this.blockSize];
      this.mac.doFinal(var1, 0);

      byte[] var3;
      for(int var2 = 0; var2 < (var3 = this.macBlock).length; ++var2) {
         var3[var2] = (byte)(this.nonceMac[var2] ^ this.associatedTextMac[var2] ^ var1[var2]);
      }

   }

   private int process(byte b, byte[] out, int outOff) {
      byte[] var4 = this.bufBlock;
      int var5;
      int var6;
      int var10000 = var6 = (var5 = this.bufOff) + 1;
      this.bufOff = var6;
      var4[var5] = (byte)b;
      if (var10000 == var4.length) {
         if (this.forEncryption) {
            b = this.cipher.processBlock(var4, 0, out, outOff);
            this.mac.update(out, outOff, this.blockSize);
         } else {
            this.mac.update(var4, 0, this.blockSize);
            b = this.cipher.processBlock(this.bufBlock, 0, out, outOff);
         }

         EAXBlockCipher var10001 = this;
         int var7;
         this.bufOff = var7 = this.blockSize;
         byte[] var8 = var10001.bufBlock;
         System.arraycopy(var8, var7, var8, 0, var7);
         return b;
      } else {
         return 0;
      }
   }

   private boolean verifyMac(byte[] mac, int off) {
      for(int var3 = 0; var3 < this.macSize; ++var3) {
         if (this.macBlock[var3] != mac[off + var3]) {
            return false;
         }
      }

      return true;
   }

   public String getAlgorithmName() {
      return this.cipher.getUnderlyingCipher().getAlgorithmName() + "/EAX";
   }

   public BlockCipher getUnderlyingCipher() {
      return this.cipher.getUnderlyingCipher();
   }

   public int getBlockSize() {
      return this.cipher.getBlockSize();
   }

   public void init(boolean forEncryption, CipherParameters params) {
      this.forEncryption = forEncryption;
      Object var3;
      byte[] forEncryption1;
      ParametersWithIV forEncryption2;
      byte[] params1;
      if (params instanceof AEADParameters) {
         AEADParameters var10000 = (AEADParameters)params;
         AEADParameters var10002 = (AEADParameters)params;
         forEncryption1 = ((AEADParameters)params).getNonce();
         params1 = var10002.getAssociatedText();
         this.macSize = var10002.getMacSize() / 8;
         var3 = var10000.getKey();
      } else {
         if (!(params instanceof ParametersWithIV)) {
            throw new IllegalArgumentException("invalid parameters passed to EAX");
         }

         ParametersWithIV var11 = forEncryption2 = (ParametersWithIV)params;
         forEncryption1 = forEncryption2.getIV();
         params1 = new byte[0];
         this.macSize = this.mac.getMacSize() / 2;
         var3 = var11.getParameters();
      }

      byte[] var4;
      byte[] var13 = var4 = new byte[this.blockSize];
      this.mac.init((CipherParameters)var3);
      int var5;
      var4[(var5 = this.blockSize) - 1] = 1;
      this.mac.update(var4, 0, var5);
      this.mac.update(params1, 0, params1.length);
      this.mac.doFinal(this.associatedTextMac, 0);
      int params2;
      var4[(params2 = this.blockSize) - 1] = 0;
      this.mac.update(var4, 0, params2);
      this.mac.update(forEncryption1, 0, forEncryption1.length);
      this.mac.doFinal(this.nonceMac, 0);
      int forEncryption3;
      var13[(forEncryption3 = this.blockSize) - 1] = 2;
      this.mac.update(var4, 0, forEncryption3);
      SICBlockCipher var12 = this.cipher;
      forEncryption2 = new ParametersWithIV.<init>((CipherParameters)var3, this.nonceMac);
      var12.init(true, forEncryption2);
   }

   public void reset() {
      this.cipher.reset();
      this.mac.reset();
      this.bufOff = 0;
   }

   public int processByte(byte in, byte[] out, int outOff) {
      return this.process(in, out, outOff);
   }

   public int processBytes(byte[] in, int inOff, int len, byte[] out, int outOff) {
      int var6 = 0;

      for(int var7 = 0; var7 != len; ++var7) {
         byte var9 = in[inOff + var7];
         int var8 = outOff + var6;
         var6 += this.process(var9, out, var8);
      }

      return var6;
   }

   public int doFinal(byte[] out, int outOff) {
      int var3 = this.bufOff;
      byte[] var4;
      byte[] var5 = new byte[(var4 = this.bufBlock).length];
      this.bufOff = 0;
      int var6;
      if (this.forEncryption) {
         EAXBlockCipher var10001 = this;
         this.cipher.processBlock(var4, 0, var5, 0);
         var4 = this.bufBlock;
         var6 = this.blockSize;
         this.cipher.processBlock(var4, var6, var5, var6);
         System.arraycopy(var5, 0, out, outOff, var3);
         this.mac.update(var5, 0, var3);
         this.calculateMac();
         byte[] var10002 = this.macBlock;
         EAXBlockCipher var10003 = this;
         int var7 = outOff + var3;
         outOff = var10003.macSize;
         System.arraycopy(var10002, 0, out, var7, outOff);
         return var3 + var10001.macSize;
      } else {
         if (var3 > (var6 = this.macSize)) {
            this.mac.update(var4, 0, var3 - var6);
            this.cipher.processBlock(this.bufBlock, 0, var5, 0);
            var4 = this.bufBlock;
            var6 = this.blockSize;
            this.cipher.processBlock(var4, var6, var5, var6);
            int var8 = var3 - this.macSize;
            System.arraycopy(var5, 0, out, outOff, var8);
         }

         this.calculateMac();
         out = this.bufBlock;
         if (this.verifyMac(out, var3 - this.macSize)) {
            return var3 - this.macSize;
         } else {
            throw new InvalidCipherTextException("mac check in EAX failed");
         }
      }
   }

   public byte[] getMac() {
      int var1;
      byte[] var2;
      byte[] var10000 = var2 = new byte[var1 = this.macSize];
      System.arraycopy(this.macBlock, 0, var2, 0, var1);
      return var10000;
   }

   public int getUpdateOutputSize(int len) {
      int var2;
      return (len + this.bufOff) / (var2 = this.blockSize) * var2;
   }

   public int getOutputSize(int len) {
      return this.forEncryption ? len + this.bufOff + this.macSize : len + this.bufOff - this.macSize;
   }
}
